/* selfDHCP
 * Copyright (C) 2002-2003 Enrico "WiseLYNX" Manzini
 *
 * module: 		selfdhcp_errlib.h
 * purposes:	definitions for the error and message handling functions
 * reference:	Stevens, Unix network programming (2ed), p.922
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef __SELFDHCP_ERRLIB_H
#define __SELFDHCP_ERRLIB_H

#include <stdio.h>
#include <stdarg.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>

#include "selfdhcp_struct.h"

#define MAXLINE 4095

/* this extern int contains the flags for the error logging management, as
 * defined in selfdhcp_errlib.h */
extern int program_flags;
extern char *prog_name;

/* the following defines have been added to the "standard" errlib from UNP,
 * with the purpose of better implementing various levels of debugging messages,
 * without the overhead of introducing new int variables.
 */
#define DEBUG_MODE			0x0004
#define DEBUG_LEVEL			0x0003	/* contains the required verbosity
						 * debug level (two bits) */
#define IS_DAEMON_PROC		0x0008	/* 1==daemon 0==non-daemon */
#define VERBOSE_MODE		0x0010
#define RESET_NIC			0x0020
#define PROFILE_ENABLED		0x0040
#define PROFILE_UPDATE		0x0080
#define TEXTONLY_MODE		0x0100
#define ARP_POISON_ENA		0x0200
#define ACTIVE_MODE			0x0400
#define USE_SMB_BCAST		0x0800

/* function display_options()
 * to be used in debug mode to show which options are set. Does not take
 * arguments because the flag variable has global scope.
 */
void display_options (void);

/* function err_msg()
 * this function has been slightly modified from the original in UNP, so that
 * it can use different "levels" of error messages, including verbose messages
 * and debug informations */
void err_msg (int level, const char *fmt, ...);

/* defines to be used as levels calling err_msg() */
#define MSG_ERR			4
#define MSG_INFO		5
#define MSG_WARN		6
/* debug messages are checked agains the required level of debug verbosity */
#define DBG_INFO		0	/* program functioning */
#define DBG_PROG		1	/* more program functioning */
#define DBG_LIB			2	/* internals (mostly structure handling) */
#define DBG_LOWLEV		3	/* lowes level routines */

void err_quit (const char *fmt, ...);

void err_ret (const char *fmt, ...);

void err_sys (const char *fmt, ...);

void err_dump (const char *fmt, ...);

#endif
