/* selfDHCP
 * Copyright (C) 2002-2003 Enrico "WiseLYNX" Manzini
 *
 * module:		selfdhcp_pthread.h
 * purposes:	definitions for the thread-oriented implementation (including
 * 				wrapped pthread_*() functions)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef __SELFDHCP_PTHREAD_H
#define __SELFDHCP_PTHREAD_H

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <syslog.h>
#include <signal.h>
#include <pthread.h>

#define _REENTRANT

#include "selfdhcp_errlib.h"
#include "selfdhcp_struct.h"
#include "selfdhcp_profiles.h"

/* semaphores for the readers 'n writers thread-safe implementation */
#define SEM_READERS		0
#define SEM_WRITERS		1
#define SEM_MESC		2
#define SEM_MESC1		3
#define SEM_MESC2		4

/* used to identify the types of struct in the semaphores functions */
#define LISTA_T			0
#define SUBLIST_T		1
#define NETCONF_T		2
#define IPLIST_T		3

/* max number of file descriptors to close if in daemon mode*/
#define MAX_FD			64

/* extern variables passed from the main function */
extern int program_flags;
extern char *prog_name;

/* function daemon_init()
 * used to initialise the daemon. Almost as in Stevens UNP.
 */
int daemon_init (void);

/* function semaphores_init()
 * used to initialise the semaphores within a given struct. type must be chosen
 * within the appropriate #define.
 */
void semaphores_init (void *toinit, int type);

/* function read_reserv()
 * used to reserv one item (passed through tolock) of a list for read. Type must
 * be chosen within the appropriate #define. If all was right, 0 is returned,
 * -1 else.
 */
int read_reserv (void *tolock, int type);

/* function read_release()
 * used to release one item (passed through tounlock) of a list previously
 * locked through read_reserv(). Type must be chosen within the appropriate
 * #define. If all was right, 0 is returned,-1 else.
 */
int read_release (void *tounlock, int type);

/* function write_reserv()
 * used to reserv one item (passed through tolock) of a list for write. Type
 * must be chosen within the appropriate #define. If all was right, 0 is
 * returned, -1 else.
 */
int write_reserv (void *tolock, int type);

/* function write_release()
 * used to release one item (passed through tounlock) of a list previously
 * locked through write_reserv(). Type must be chosen within the appropriate
 * #define. If all was right, 0 is returned,-1 else.
 */
int write_release (void *tounlock, int type);

/* wrapped pthread_mutex_lock() */
int Pthread_mutex_lock (pthread_mutex_t * __mutex);

/* wrapped pthread_mutex_unlock() */
int Pthread_mutex_unlock (pthread_mutex_t * __mutex);

/* wrapped pthread_create() */
int Pthread_create (pthread_t * thread, pthread_attr_t * attr,
		    void *(*start_routine) (void *), void *arg);

#endif
