/* selfDHCP
 * Copyright (C) 2002-2003 Enrico "WiseLYNX" Manzini
 *
 * module:		main.c
 * purposes:	main source file
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <popt.h>
#include <sys/types.h>
#include <unistd.h>
#include <syslog.h>
#include <pthread.h>
#include <errno.h>

#define _REENTRANT

#include "selfdhcp_errlib.h"
#include "selfdhcp_pthread.h"
#include "selfdhcp_struct.h"
#include "selfdhcp_capture.h"
#include "selfdhcp_profiles.h"
#include "selfdhcp_heuristics.h"

#define SELFDHCP_VERSION	PACKAGE_VERSION

/* these two are global variables */
int program_flags = USE_SMB_BCAST;
char *prog_name;

/* used for error handling; have a look at 'man errno' */
extern int errno;

/* function main() */
int
main (int argc, char **argv)
{
  char *profile_fname = NULL, *iface_name = NULL, *mac_addr = NULL;
  char *config_fname = NULL, *test_IP_char = NULL;
  char *tmp, opt_count;
  unsigned char mac_addr_num[ETH_ALEN];
  struct in_addr test_IP_num;
  uint32_t default_test_IPs[4] = { DEFAULT_TEST_IP };
  lista_t **liste;
  netconfig_t net_conf[3];
  capture_args_t cap_args;
  ip2test_t iplist;
  arpp_args_t arpp_args;
#ifndef NEW_LIBNET_INTERFACE
  struct libnet_link_int net_link;
#endif
  test_hosts_t hosts;
//      void *args[3];
  int capture_timeout = 0, debug_level = -1;
  pthread_t cap_thread, arpp_thread;
  poptContext popt_context;
/* these are the options selfdhcp can get from command line. Have a look at
 * man popt for explaination about the use of popt. */
  struct poptOption popt_opt[] = {
/* arp flooding */
    {"arpflood", 'a', POPT_ARG_NONE, NULL, 'a',
     "Use arp flooding, used on switched networks"},
/* active mode */
    {"active", 'A', POPT_ARG_NONE, NULL, 'A', "Actively scan known networks"},
/* plain text mode */
    {"consoleoutput", 'c', POPT_ARG_NONE, NULL, 'c',
     "Don't do any configuration, just write to stdout"},
/* configuration file */
    {"config", 'C', POPT_ARG_STRING | POPT_ARGFLAG_OPTIONAL, config_fname,
     'C',
     "Use conf_fname as configuration file"
     " (default " DEFAULT_CONFIG_FILE ")", "conf_fname"},
/* wether or not we are in daemon mode */
    {"daemon", 'd', POPT_ARG_NONE, NULL, 'd', "Daemon mode"},
/* debug logging and messages */
    {"debug", 'D', POPT_ARG_INT | POPT_ARGFLAG_OPTIONAL, &debug_level, 0,
     "Displays debug messages, with various levels of detail (within [1-4]"
     ", default 1)", "level"},
/* interface to be used */
    {"iface", 'i', POPT_ARG_STRING, iface_name, 'i',
     "Interface to be used", "device"},
/* host to contact for testing configuration */
    {"host", 'H', POPT_ARG_STRING, test_IP_char, 'H', "Host to be used for "
     "testing configurations", "IP"},
/* MAC changing */
    {"mac", 'm', POPT_ARG_STRING, mac_addr, 'm',
     "Change MAC addr to new_mac (or"
     " to a random one if new_mac is 'rand') before running", "new_mac"},
/* wether we sould use a profile set or not, and optionally the profile file */
    {"profile", 'p', POPT_ARG_STRING | POPT_ARGFLAG_OPTIONAL, profile_fname,
     'p',
     "Use prof_fname as profile source (default " DEFAULT_PROFILE_FILE ")",
     "prof_fname"},
/* reset */
    {"reset", 'r', POPT_ARG_NONE, NULL, 'r',
     "Reset the NIC, as with 'ifconfig iface 0.0.0.0'"},
/* timeout */

    {"timeout", 't', POPT_ARG_INT, &capture_timeout, 't',
     "Capture timeout (minutes) without receiving packets (if not specified"
     " with this option, it is assumed to be 5 min)", "timeout"},
/* profiles file update */
    {"update", 'u', POPT_ARG_NONE, NULL, 'u', "Enable profiles file update"},
/* verbose mode */
    {"verbose", 'v', POPT_ARG_NONE, NULL, 'v', "Verbose mode"},
/* version display */
    {"version", 'V', POPT_ARG_NONE, NULL, 'V', "Display version"},
/* automatic help and usage, from popt */
    POPT_AUTOHELP {NULL, 0, 0, NULL, 0}
  };

/* after creating a popt option context, loop until all options are acquired */
  popt_context = poptGetContext (NULL, argc, argv, popt_opt, 0);
  while ((opt_count = poptGetNextOpt (popt_context)) >= 0)
    {
      switch (opt_count)
	{
	case 'a':
	  program_flags = program_flags | ARP_POISON_ENA;
	  break;
	case 'A':
	  program_flags = program_flags | ACTIVE_MODE;
	  break;
	case 'c':
	  program_flags = program_flags | TEXTONLY_MODE;
	  break;
	case 'C':
	  config_fname = (char *) poptGetOptArg (popt_context);
	  if (NULL == config_fname)
	    config_fname = strdup (DEFAULT_CONFIG_FILE);
	  break;
	case 'd':
	  program_flags = program_flags | IS_DAEMON_PROC;
	  break;
	case 'D':
	  program_flags = program_flags | DEBUG_MODE;
	  break;
	case 'H':
	  test_IP_char = (char *) poptGetOptArg (popt_context);
	  break;
	case 'i':
	  iface_name = (char *) poptGetOptArg (popt_context);
	  break;
	case 'm':
	  mac_addr = (char *) poptGetOptArg (popt_context);
	  break;
	case 'p':
	  program_flags = program_flags | PROFILE_ENABLED;
	  profile_fname = (char *) poptGetOptArg (popt_context);
	  break;
	case 'r':
	  program_flags = program_flags | RESET_NIC;
	  break;
	case 'u':
	  program_flags = program_flags | PROFILE_UPDATE;
	  break;
	case 'v':
	  program_flags = program_flags | VERBOSE_MODE;
	  break;
	case 'V':
	  printf ("selfdhcp Version %s\n", SELFDHCP_VERSION);
	  return 0;
	default:
	  break;
	}
    }
/* we put in prog_name the program name without the path */
  prog_name = argv[0];
  while (NULL != (tmp = strstr (prog_name, "/")))
    prog_name = tmp + 1;
/* if an error occurred while option processing... */
  if (opt_count < -1)
    err_quit ("%s: %s", poptBadOption (popt_context, POPT_BADOPTION_NOALIAS),
	      poptStrerror (opt_count));
  poptFreeContext (popt_context);
/* setting the debug level (with checks for the range of value). We have to set
 * this before any other, because debug level is required within err_msg(),
 * which is used throught the whole program for message handling */
  if (debug_level != -1)
    program_flags += DEBUG_MODE;
  if (program_flags & DEBUG_MODE)
    {
      if (debug_level == 0)
	debug_level = 1;
      if (debug_level < 1 || debug_level > 4)
	err_quit ("wrong debug level %d, should be [1-4]", debug_level);
      else
	program_flags = program_flags | (debug_level - 1);
    }
/* if a configuration file was provided, it's time to read it */
  debug_level = 0;
  if (NULL != config_fname)
/* in debug_level is stored the return value of selfconf_read(), so that later
 * if the file did not exist we can create it */
    debug_level = selfconf_read (config_fname, iface_name, mac_addr,
				 &profile_fname, &capture_timeout);
  if (program_flags & DEBUG_MODE)
    display_options ();
/* if in daemon mode we need further initialization */
  if (program_flags & IS_DAEMON_PROC)
    {
      if (-1 == daemon_init ())
	err_sys ("daemon_init() failed");
      err_msg (DBG_PROG, "daemon initialized");
    }
/* initializing the MAC-IP lists and the net configuration array */
  liste = createlists ();
  netconf_init (net_conf);
/* setting defaults if not specified from parameters */
  if (iface_name == NULL)
    iface_name = strdup (DEFAULT_IFACE);
  if (test_IP_char != NULL)
    {
      if (0 == inet_aton (test_IP_char, &test_IP_num))
	{
	  err_msg (MSG_ERR, "%s is not a valid IP address, using default");
	  hosts.test_IPs = default_test_IPs;
	  hosts.num = DEFAULT_TEST_IP_NUM;
	}
      err_msg (DBG_INFO, "using %s as test host", inet_ntoa (test_IP_num));
      hosts.test_IPs = (uint32_t *) & test_IP_num;
      hosts.num = 1;
    }
  else
    {
      err_msg (DBG_INFO, "using default test hosts");
      hosts.test_IPs = (struct in_addr *) default_test_IPs;
      hosts.num = DEFAULT_TEST_IP_NUM;
    }
  if (capture_timeout == 0)
    capture_timeout = DEFAULT_CAPTURE_TIMEOUT;
  err_msg (MSG_INFO, "interface to use is %s", iface_name);
  err_msg (DBG_INFO, "heuristics timeout set to %d minutes", capture_timeout);
/* setting timeout in seconds */
  capture_timeout *= 60;
/* handling the profile file stuff */
  if (program_flags & PROFILE_ENABLED)
    {
      if (profile_fname == NULL)
	profile_fname = strdup (DEFAULT_PROFILE_FILE);
      opt_count = xml_read_profiles (profile_fname, net_conf);
      if (-1 != opt_count)
	err_msg (MSG_INFO, "profile file %s opened, %d profiles read",
		 profile_fname, opt_count);
    }
/* setting PROFILE_UPDATE and not PROFILE_ENABLED is a bit nonsense... */
  if (program_flags & PROFILE_UPDATE && !(program_flags & PROFILE_ENABLED))
    {
      err_msg (MSG_WARN,
	       "--update option selected without --profile option..."
	       " unsetting");
      program_flags &= (~PROFILE_UPDATE);
    }
/* idem for ACTIVE_MODE */
  if (program_flags & ACTIVE_MODE && !(program_flags & PROFILE_ENABLED))
    {
      err_msg (MSG_WARN,
	       "--active option selected without --profile option..."
	       " unsetting");
      program_flags &= (~ACTIVE_MODE);
    }
/* because almost all is set, if we have to write down a configuration file
 * this is the right point */
  if (-1 == debug_level)
    {
      err_msg (MSG_INFO, "writing down configuration to %s", config_fname);
      selfconf_write (config_fname, iface_name, mac_addr,
		      profile_fname, &capture_timeout);
      err_msg (MSG_INFO, "configuration written, exiting");
      return 0;
    }
/* reset of the NIC if required */
  if (program_flags & RESET_NIC)
    netconf_unapply (iface_name);
/* changing the MAC if required */
  if (mac_addr != NULL)
    {
      if (0 == strncmp (mac_addr, "rand", 5))
	{
	  rand_mac (mac_addr_num);
	  if (NULL == (mac_addr = malloc (3 * ETH_ALEN)))
	    err_quit ("malloc(): %s", strerror (errno));
	  mac2str (mac_addr_num, mac_addr);
	}
      err_msg (DBG_INFO, "new MAC will be %s", mac_addr);
      change_mac (mac_addr, iface_name);
    }
/* preparing arguments to be passed as void* */
  arpp_args.device = iface_name;
#ifndef NEW_LIBNET_INTERFACE
  arpp_args.net_link = &net_link;
#endif
  arpp_args.tosleep = SWITCH_TIMEOUT;
  cap_args.device = iface_name;
  cap_args.liste = liste;
  cap_args.net_conf = net_conf;
  cap_args.iplist = &iplist;
  err_msg (DBG_PROG, "initialization finished, entering the program");
/* we create a new thread for the arp flooding and capture process */
  if (program_flags & ARP_POISON_ENA)
    {
      arpp_init (&arpp_args);
      err_msg (DBG_PROG, "starting arp flooding thread");
      Pthread_create (&arpp_thread, NULL, (void *) &arpp_doit, &arpp_args);
    }
  err_msg (DBG_PROG, "starting packet capture thread");
  Pthread_create (&cap_thread, NULL, (void *) &capture_start,
		  (void *) &cap_args);
/* heuristics runs in the main thread */
  heuristics_start (iface_name, liste, capture_timeout, &iplist, net_conf,
		    &hosts);
  if (program_flags & PROFILE_UPDATE)
    err_msg (MSG_INFO, "%d profiles written to file %s", profile_fname,
	     write_profiles (profile_fname, &(net_conf[NETCONF_PROFILES])));
  return 0;
}
